var fs = require('fs')
var path = require('path')
var testCase = require('nodeunit').testCase

require('utils/FileStorage.js')

var directory = 'tmp'

GenerateFiles = function(directory) {

  for (var index = 1; index < 4; index++) {
    fs.writeFileSync(path.join(directory, 'f' + index), '')
  }
  
}

GenerateSample = function(directory) {
  
  var storage = new FileStorage(directory)
  storage.add('iPhone', { company: 'Apple', type: 'mobile' })
  storage.add('Galaxy', { company: 'Samsung', type: 'mobile' })
  storage.add('Lumix', { company: 'Panasonic', type: 'camera' })
  
  return storage
  
}

module.exports = testCase({
  setUp: function (callback) {
    FileStorage.rmdir(directory)
    callback()
  },
  tearDown: function (callback) {
    FileStorage.rmdir(directory)
    callback()
  },
  // Test class methods
  test_rmdir: function (test) {
    FileStorage.mkdir(directory)
    GenerateFiles(directory)
    
    var subLevels = ['a', 'b', 'c']
    for (var index in subLevels) {
      var subDirectory = path.join(directory, subLevels[index])
      FileStorage.mkdir(subDirectory)
      GenerateFiles(subDirectory)
      
      for (var index in subLevels) {
        var thirdSubDirectory = path.join(subDirectory, subLevels[index])
        FileStorage.mkdir(thirdSubDirectory)
        GenerateFiles(thirdSubDirectory)
      }
    }
    
    FileStorage.rmdir(directory, function() {
      test.equals(path.existsSync(directory), false, "Directory shouldn't exists")
      test.done()
    })

  },
  // Test object methods
  test_add: function(test) {
    var storage = GenerateSample(directory)
    
    var children = fs.readdirSync(directory)
    test.equals(children.length, 3, 'Sub directory should equals to total object')
    
    test.done()
  },
  test_remove: function(test) {
    var storage = GenerateSample(directory)
    storage.remove('iPhone')
    storage.remove('Galaxy')
    
    var children = fs.readdirSync(directory)
    var found = 0
    for (var index in children) {
      var child = path.join(directory, children[index])
      var hashFiles = fs.readdirSync(child)
      if (hashFiles.length > 0) found++
    }

    test.equals(found, 1, 'After remove two key, it should found only one')
    test.done()
  },
  test_update: function(test) {
    var storage = GenerateSample(directory)
    storage.update('iPhone', { company: 'Apple', type: 'mobile', update: true })
    
    var children = fs.readdirSync(directory)
    for (var index in children) {
      var files = fs.readdirSync(path.join(directory, children[index]))
      for (var subindex in files) {
        if (files[subindex] == 'iPhone.key') {
          var file = path.join(path.join(directory, children[index]), files[subindex])
          var data = fs.readFileSync(file)
          
          test.equals(JSON.parse(data).update, true, 'iPhone should have new property')
          test.done()
        }
      }
    }
    
  },
  test_get: function(test) {
    var storage = GenerateSample(directory)
    
    test.equals(storage.get('iPhone').company, 'Apple' )
    test.equals(storage.get('Galaxy').type, 'mobile' )
    
    test.throws(function() { storage.get('notfound') })
    
    test.done()
  }
})