var testCase = require('nodeunit').testCase
var util = require('util')

require('models/User.js')
require('utils/predicates/AggregatePredicate.js')
require('utils/predicates/RegexPredicate.js')
require('utils/query/users/PredicateQuery.js')
require('utils/FileStorage.js')
require('utils/UserFileStorage.js')

var counter = 0
var storage = null

module.exports = testCase({
  setUp: function (callback) {
    storage = new UserFileStorage('tmp' + counter)
    callback()
  },
  tearDown: function (callback) {
    counter++
    storage.clean()
    storage = null
    callback()
  },
  // Test object methods
  test_add: function(test) {
    storage.add(User.create('user1', 'password1'))
    storage.count(function(err, count) {
      test.equal(count, 1)
    })
    
    storage.add(User.create('user1', 'duplicate'), function(err) {
      test.ok(err)
    })
    storage.count(function(err, count) {
      test.equal(count, 1)
    })
    
    storage.add(User.create('user2', 'password2'))
    storage.count(function(err, count) {
      test.equal(count, 2)
    })
    
    storage.get('user2', function(err, user) {
      test.ok(user.authenticate('password2'))
      test.ok(!user.authenticate('wrong password'))
    })
        
    test.done()
  },
  test_remove: function(test) {
    storage.add(User.create('user1', 'password'))
    storage.add(User.create('user2', 'password'))
    storage.add(User.create('user3', 'password'))
    storage.count(function(err, count) {
      test.equal(count, 3)
    })
    
    storage.remove('user2')
    storage.count(function(err, count) {
      test.equal(count, 2)
    })
    
    storage.get('user2', function(err, user) {
      test.ok(err)
    })
    storage.get('user1', function(err, user) {
      test.ok(user)
    })
    storage.get('user3', function(err, user) {
      test.ok(user)
    })
    
    test.done()
  },
  test_list: function(test) {
    storage.add(User.create('steaveroger', 'password'))
    storage.add(User.create('mariahill', 'password'))
    storage.add(User.create('buckybarnes', 'password'))
    storage.add(User.create('brianbraddock', 'password'))
    storage.add(User.create('clintbarton', 'password'))
    storage.add(User.create('tonystark', 'password'))
    storage.add(User.create('nohvarr', 'password'))
    storage.add(User.create('thaddeus', 'password'))
    storage.add(User.create('peterparker', 'password'))
    storage.add(User.create('jessicadrew', 'password'))
    storage.add(User.create('thorodinson', 'password'))
    storage.add(User.create('jameshowlett', 'password'))
    storage.count(function(err, count) {
      test.equal(count, 12)
    })
    
    var predicate1 = new RegexPredicate('username', /^b(.*)$/)
    var predicateQuery = new PredicateQuery(predicate1, function(usernames) {
      test.equal(usernames.length, 2)
    })
    storage.list(predicateQuery)
    
    var predicate2 = new RegexPredicate('username', /^j(.*)$/)
    predicateQuery = new PredicateQuery(predicate2, function(usernames) {
      test.equal(usernames.length, 2)
    })
    storage.list(predicateQuery)
    
    var predicates = new AggregatePredicate([predicate1, predicate2])
    predicateQuery = new PredicateQuery(predicates, function(usernames) {
      test.equal(usernames.length, 4)
      test.done()
    })
    storage.list(predicateQuery)
  }
})