var testCase = require('nodeunit').testCase
var util = require('util')

require('models/User.js')
require('utils/predicates/AggregatePredicate.js')
require('utils/predicates/RegexPredicate.js')
require('utils/UserHbaseStorage.js')
require('utils/query/users/FilterQuery.js')

var storage = null

module.exports = testCase({
  setUp: function (callback) {
    storage = new UserHbaseStorage('labs.local', 8080, 'users')
    storage.init(function(success) {
      callback()
    })
  },
  tearDown: function (callback) {
    storage.drop(function(success) {
          storage = null
          callback()
        })
  },
  // Test object methods
  test_add: function(test) {
    
    var validate4 = function(err, user) {
      test.ok(user.authenticate('password1'))
      test.ok(!user.authenticate('wrong password'))
      
      test.done()
    }
    
    var validate3 = function(err) {
      storage.count(function(err, count) {
        test.equal(count, 2)
        storage.get('user1', validate4)
      })
    }
    
    var validate2 = function(err) {
      test.ok(err)
      
      storage.count(function(err, count) {
        test.equal(count, 1)
        storage.add(User.create('user2', 'password2'), validate3)
      })
    }
    
    var validate1 = function(err) {
      storage.count(function(err, count) {
        test.equal(count, 1)
        storage.add(User.create('user1', 'password2'), validate2)
      })
    }
    
    storage.add(User.create('user1', 'password1'), validate1)
    
  },
  test_remove: function(test) {
    var invoke = 0
    
    var validate4 = function(err, user) {
      invoke++
      test.ok(user)
      test.ok(!err)
      if (invoke == 2) {
        test.done()
      }
    }
    
    var validate3 = function(err, user) {
      test.ok(err)
      invoke = 0
      storage.get('user1', validate4)
      storage.get('user3', validate4)
    }
    
    var validate2 = function(err) {
      storage.count(function(err, count) {
        test.equal(count, 2)
        storage.get('user2', validate3)
      })
    }
    
    var validate1 = function(err) {
      invoke++
      
      if (invoke == 3) {
        storage.count(function(err, count) {
          test.equal(count, 3)
          storage.remove('user2', validate2)
        })
      }
    }
    
    storage.add(User.create('user1', 'password'), validate1)
    storage.add(User.create('user2', 'password'), validate1)
    storage.add(User.create('user3', 'password'), validate1)
    
  },
  test_list: function(test) {
    var invoke = 0
    
    var filter1 = {
      "op": "EQUAL",
      "type": "RowFilter",
      "comparator": {"value": "^b.+$", "type": "RegexStringComparator"}
    }
    var filter2 = {
      "op": "EQUAL",
      "type": "RowFilter",
      "comparator": {"value": "^j.+$", "type": "RegexStringComparator"}
    }
    var filter3 = {
      "op": "MUST_PASS_ONE",
      "type": "FilterList",
      "filters": [ filter1, filter2 ]
    }
    
    var validate4 = function(results) {
      test.equal(results.length, 4)
      test.done()
    }
    
    var validate3 = function(results) {
      test.equal(results.length, 2)
      
      var filterQuery = new FilterQuery({ filter: filter3 }, validate4)
      storage.list(filterQuery)
    }
    
    var validate2 = function(results) {
      test.equal(results.length, 2)
      
      var filterQuery = new FilterQuery({ filter: filter2 }, validate3)
      storage.list(filterQuery)
    }
    
    var validate1 = function(err) {
      invoke++
      if (invoke == 12) {
        invoke = 0
        var filterQuery = new FilterQuery({ filter: filter1 }, validate2)
        storage.list(filterQuery)
      }
    }
    
    storage.add(User.create('steaveroger', 'password'), validate1)
    storage.add(User.create('mariahill', 'password'), validate1)
    storage.add(User.create('buckybarnes', 'password'), validate1)
    storage.add(User.create('brianbraddock', 'password'), validate1)
    storage.add(User.create('clintbarton', 'password'), validate1)
    storage.add(User.create('tonystark', 'password'), validate1)
    storage.add(User.create('nohvarr', 'password'), validate1)
    storage.add(User.create('thaddeus', 'password'), validate1)
    storage.add(User.create('peterparker', 'password'), validate1)
    storage.add(User.create('jessicadrew', 'password'), validate1)
    storage.add(User.create('thorodinson', 'password'), validate1)
    storage.add(User.create('jameshowlett', 'password'), validate1)
  }
})