var fs = require('fs')
var path = require('path')
var crypto = require('crypto')
var util = require('util')

require('core/Storage.js')

FileStorage = function(directory) {
  Storage.call(this)
  FileStorage.mkdir(directory)
  
  this.directory = directory
  
  this.hash = function(directory) {
    var hex = crypto.createHash('sha256').update(directory).digest('hex')
    var sum = 0
    for (var index in hex) {
      sum += hex.charCodeAt(index)
    }
    
    var mod = 'a'.charCodeAt(0) + ( sum % 26 )
    
    return String.fromCharCode(mod)
  }
  
}

util.inherits(FileStorage, Storage)

FileStorage.mkdir = function(directory) {
  
  if (!path.existsSync(directory)) {
    fs.mkdirSync(directory, 0755)
  }
  
}

FileStorage.rmdir = function(directory, callback) {
  if (!path.existsSync(directory)) return
  
  var rootDirectory = directory
  var currentDirectory = directory
  var removed = false
  
  fs.readdir(directory, function(err, files) {

    if (files == undefined) { return }
    
    var children = files
    while (children.length > 0) {
      
      var file = children[0]
      var filePath = path.join(currentDirectory, file)

      if (path.existsSync(filePath)) {
        var stat = fs.statSync(filePath)

        if (stat.isDirectory()) {
          currentDirectory = filePath
        } else {
          fs.unlinkSync(filePath)
        }
        
        children = fs.readdirSync(currentDirectory)
      } else {
        removed = true
      }
      
      while (children.length == 0) {
        fs.rmdirSync(currentDirectory)
        
        if (currentDirectory == rootDirectory) {
          removed = true
          break
        }
        currentDirectory = path.dirname(currentDirectory)
        children = fs.readdirSync(currentDirectory)
      }
      
      if (removed) break
      
    }

    if (path.existsSync(currentDirectory)) {
      fs.rmdirSync(currentDirectory)
    }
    
    if (callback) {
      callback()
    }
    
  })
  
}

FileStorage.prototype = {
  add: function(key, object) {
    var targetDirectory = path.join(this.directory, this.hash(key))
    FileStorage.mkdir(targetDirectory)
    
    var targetFile = path.join(targetDirectory, key + '.key')
    fs.writeFileSync(targetFile, JSON.stringify(object))
  },
  remove: function(key) {
    var targetDirectory = path.join(this.directory, this.hash(key))
    var targetFile = path.join(targetDirectory, key + '.key')
    
    if (path.existsSync(targetFile)) {
      fs.unlinkSync(targetFile)
    }
  },
  update: function(key, object) {
    var targetDirectory = path.join(this.directory, this.hash(key))
    var targetFile = path.join(targetDirectory, key + '.key')
    if (path.existsSync(targetFile)) {
      fs.unlinkSync(targetFile)
      fs.writeFileSync(targetFile, JSON.stringify(object))
    } else {
      throw new Error('Not Found')
    }
  },
  get: function(key) {
    var targetDirectory = path.join(this.directory, this.hash(key))
    var targetFile = path.join(targetDirectory, key + '.key')
    
    if (path.existsSync(targetFile)) {
      var data = fs.readFileSync(targetFile)
      return JSON.parse(data)
    } else {
      throw new Error('Not Found')
    } 
  }
}