var util = require('util')

require('core/UserStorage.js')
require('utils/FileStorage.js')
require('utils/query/users/PredicateQuery.js')


UserFileStorage = function(directory) {
  UserStorage.call(this)
  
  var _storage = new FileStorage(directory)
  this.storage = _storage
  
  this.clean = function() {
    FileStorage.rmdir(_storage.directory)
  }
  
  this._query = function(query) {
    query.caller = 'file'
    query.storage = _storage
  }
  
}

util.inherits(UserFileStorage, UserStorage)

UserFileStorage.prototype.add = function(user, callback) {
  if (!this._validateUser(user)) return
  
  var usernames = null
  try {
    usernames = this.storage.get('list')
  } catch (e) {
    usernames = {}
    this.storage.add('list', usernames)
  }
  
  if (!usernames[user.username.toLowerCase()]) {
    usernames[user.username.toLowerCase()] = true
    this.storage.update('list', usernames)
    this.storage.add(user.username, user)
    
    if (callback) callback(null)
  } else {
    if (callback) callback(new Error('User already exists'))
  }
}
UserFileStorage.prototype.remove = function(username, callback) {
  if (!this._validateUsername(username)) return
  
  var usernames = null
  try {
    usernames = this.storage.get('list')
  } catch (e) {
    usernames = {}
  }
  
  if (usernames[username.toLowerCase()]) {
    delete usernames[username]
    this.storage.update('list', usernames)
    this.storage.remove(username)
    
    if (callback) callback(null)
  } else {
    if (callback) callback(new Error('User not found'))
  }
}
UserFileStorage.prototype.count = function(callback) {
  var _count = 0
  
  try {
    var usernames = this.storage.get('list')
    for (var key in usernames) {
      if (usernames[key]) _count++
    }
  } catch (e) {
  }
  
  if (callback) callback(null, _count)
}

