var util = require('util'),
    hbase = require('hbase')

require('core/UserStorage.js')

UserHbaseStorage = function(address, port, tableName, tablePrefix) {
  UserStorage.call(this)
  
  this.client = hbase({ host: address, port: port})
  
  var prefix = 'ixor_'
  if (tablePrefix) prefix = tablePrefix + '_'
  
  var _table = this.client.getTable(prefix + tableName)
  this.table = _table
  
  this._query = function(query) {
    query.caller = 'hbase'
    query.scanner = _table.getScanner()
  }
}

util.inherits(UserHbaseStorage, UserStorage)

UserHbaseStorage.prototype.add = function(user, callback) {
  if (!this._validateUser(user)) {
    if (callback) callback(new Error('Something wrong with user object'))
    return
  }
  
  var _user = this.table.getRow(user.username)
  _user.exists(function(err, exists) {
    if (!exists) {
      _user.put([{column: 'user:password', $:user.password}], function(err, success) {
        if (callback) callback(null)
      })
    } else {
      if (callback) callback(new Error('User already exists'))
    }
  })
}
UserHbaseStorage.prototype.remove = function(username, callback) {
  if (!this._validateUsername(username)) {
    if (callback) callback(new Error('Something wrong with username'))
    return
  }
  
  var _user = this.table.getRow(username)
  _user.exists(function(err, exists) {
    if (!exists) {
      if (callback) callback(new Error('User not found'), null)
    } else {
      
      _user.delete(function(err, success) {
        if (callback) callback(null)
      })
      
    }
  })
}
UserHbaseStorage.prototype.update = function(username, user, callback) {
  if (!this._validateUsername(username)) {
    if (callback) callback(new Error('Something wrong with username'))
    return
  }
  
  var _user = this.table.getRow(username)
  _user.exists(function(err, exists) {
    if (!exists) {
      if (callback) callback(new Error('User not found'), null)
    } else {
      
      _user.put([{column: 'user:password', $:user.password}], function(err, success) {
        if (callback) callback(null)
      })
      
    }
  })
}


UserHbaseStorage.prototype.get = function(username, callback) {
  if (!this._validateUsername(username)) {
    if (callback) callback(new Error('Something wrong with username'))
    return
  }
    
  var _user = this.table.getRow(username)
  _user.exists(function(err, exists) {
    if (!exists) {
      if (callback) callback(new Error('User not found'), null)
    } else {
        
      _user.get(function(err, columns) {
          
        var user = Object.create(User)
          
        user.username = username
        user.password = columns[0].$
          
        if (callback) callback(null, user)
          
      })
        
    }
  })
}
UserHbaseStorage.prototype.count = function(callback) { 
  var _users = this.table.getRow('*')
  _users.get(function(err, value) {
    var _count = 0
    if (value) _count = value.length
    if (callback) callback(null, _count)
  })
    
}  

// Hbase methods
UserHbaseStorage.prototype.init = function(callback) {
  var _table = this.table
  
  _table.create({
    ColumnSchema: [{ name: 'user' }, { name: 'profile' }]
  }, 
  function(err, success) { 
    if (callback) { callback(success) }
  })
}
UserHbaseStorage.prototype.schema = function(callback) {
  this.table.getSchema(function(err, schema) {
    console.log(schema)
    if(callback) callback()
  })
}
UserHbaseStorage.prototype.drop = function(callback) {
  this.table.delete(
    function(err, success) { 
      if (callback) { callback(success) }
    }
  )
}